#include <climits>
#include <algorithm>
#include "astar.hpp"

Astar::Astar(nodeMatrix* map) : start(map->at(0).at(0)), end(map->at(map->size() - 1).at(
        map->at(map->size() - 1).size() - 1)), map(map) {}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Astar::initialize() {
    path.push_back(start);
}

void Astar::findPath() {
    Node* currentNode = start;
    while (currentNode != end) {
        nodeList neighbors = findChildren(currentNode);
        int random = rand() % neighbors.size();
        currentNode = neighbors.at(random);
        path.push_back(currentNode);
    }
    std::reverse(path.begin(), path.end());
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

nodeList Astar::findChildren(Node* pNode) {
    nodeList children;
    int x = pNode->getX();
    int y = pNode->getY();
    if (x > 0) {
        children.push_back(map->at(x - 1).at(y));
    }
    if (x < map->size() - 1) {
        children.push_back(map->at(x + 1).at(y));
    }
    if (y > 0) {
        children.push_back(map->at(x).at(y - 1));
    }
    if (y < map->at(x).size() - 1) {
        children.push_back(map->at(x).at(y + 1));
    }
    if (x > 0 && y > 0) {
        children.push_back(map->at(x - 1).at(y - 1));
    }
    if (x > 0 && y < map->at(x).size() - 1) {
        children.push_back(map->at(x - 1).at(y + 1));
    }
    if (x < map->size() - 1 && y > 0) {
        children.push_back(map->at(x + 1).at(y - 1));
    }
    if (x < map->size() - 1 && y < map->at(x).size() - 1) {
        children.push_back(map->at(x + 1).at(y + 1));
    }
    for (int i = 0; i < children.size(); i++) {
        if (children[i]->getObstacle() == 1) {
            children.erase(children.begin() + i);
            i--;
        }
    }
    return children;
}

bool Astar::isNodeOnList(Node* pNode, const nodeList &vector1) {
    for (Node* node: vector1) {
        if (node->getX() == pNode->getX() && node->getY() == pNode->getY()) {
            return true;
        }
    }
    return false;
}

int Astar::findPlace(Node* pNode, const nodeList &vector) {
    for (int i = 0; i < vector.size(); i++) {
        if (vector[i] == pNode) {
            return i;
        }
    }
    return -1;
}

Node* Astar::findLowestCostNode(const nodeList &vector) {
    Node* lowestCostNode = nullptr;
    int lowestCost = INT_MAX;
    for (Node* node: vector) {
        if (node->getCostF() < lowestCost) {
            lowestCost = node->getCostF();
            lowestCostNode = node;
        }
    }
    return lowestCostNode;
}