#ifndef ASTAR_ASTAR_HPP
#define ASTAR_ASTAR_HPP
#include <vector>
#include "world.hpp"

typedef std::vector<std::vector<Node*>> nodeMatrix;
typedef std::vector<Node*> nodeList;

class Astar {
private:
    // Les nœuds start et end sont initialisés dans le constructeur, vous pouvez les utiliser pour vous aider à déterminer la meilleure route dans initialize() et findPath()
    Node* start;
    Node* end;

    // Le pointeur vers la matrice de nœuds représentant la carte
    nodeMatrix* map;
public:
    // Ces trois listes sont à remplir avec les fonctions initialize() et findPath()
    nodeList open_list;
    nodeList closed_list;
    // Le chemin trouvé est stocké dans cette liste et doit commencer par le nœud end et se terminer par le nœud start
    nodeList path;
    // Constructeur permettant d'initialiser tous les attributs précédents, à ne pas modifier
    explicit Astar(nodeMatrix* map);

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    void initialize();
    void findPath();
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    // Fonctions utilitaires déjà codées que vous pouvez utiliser si besoin
    /**
     * Trouve tous les voisins accessible d'une case (c'est à dire les 8 directions possibles en supprimant les cases non accessibles)
     * @param pNode Le pointeur vers la case dont on veut trouver les voisins
     * @return une liste de pointeurs vers les voisins de la case, vide si il n'y a pas de voisins
     */
    nodeList findChildren(Node* pNode);

    /**
     * Recherche si un noeud est dans une liste en comparant les coordonnées
     * @param pNode le pointeur vers le noeud à rechercher
     * @param vector1 le reference vers la liste dans laquelle on recherche
     * @return true si le noeud est dans la liste, false sinon
     */
    static bool isNodeOnList(Node* pNode, const nodeList& vector1);

    /**
     * Cherche la place d'un noeud dans une liste
     * @param pNode le pointeur vers le noeud à rechercher
     * @param vector la reference vers la liste dans laquelle on recherche
     * @return l'indice du noeud dans la liste, -1 si il n'y est pas
     */
    static int findPlace(Node* pNode, const nodeList& vector);

    /**
     * Recherche le noeud ayant le costF le plus faible dans une liste
     * @param vector le pointeur vers la liste dans laquelle on recherche
     * @return le pointeur vers le noeud ayant le costF le plus faible
     */
    static Node* findLowestCostNode(const nodeList& vector);
};


#endif //ASTAR_ASTAR_HPP
