#include "world.hpp"

Node::Node(int x, int y, int obstacle) : x(x), y(y), cost_g(0), cost_h(0), cost_f(0), obstacle(obstacle),
                                         parent(nullptr) {}

int Node::getX() const { return x; }

int Node::getY() const { return y; }

int Node::getCostG() const { return cost_g; }

int Node::getCostH() const { return cost_h; }

int Node::getCostF() const { return cost_f; }

int Node::getObstacle() const { return obstacle; }

void Node::setCostG(int costG) { this->cost_g = costG; }

void Node::setCostH(int costH) { this->cost_h = costH; }

void Node::setCostF(int costF) { this->cost_f = costF; }

Node* Node::getParent() { return parent; }

void Node::setParent(Node* newParent) { this->parent = newParent; }

int distance(Node* node1, Node* node2) {
    return (node2->getX() - node1->getX()) * (node2->getX() - node1->getX()) +
           (node2->getY() - node1->getY()) * (node2->getY() - node1->getY());
}

std::vector<std::vector<Node*>> createMap(int width, int height, int obstacleProbability, bool hard) {
    std::vector<std::vector<Node*>> map;
    for (int i = 0; i < height; i++) {
        std::vector<Node*> row;
        for (int j = 0; j < width; j++) {
            int obstacle = 0;
            if (rand() % 100 < obstacleProbability) {
                obstacle = 1;
            }
            row.push_back(new Node(i, j, obstacle));
        }
        map.push_back(row);
    }
    delete map.at(0).at(0);
    map.at(0).at(0) = new Node(0, 0, 0);
    delete map.at(height - 1).at(width - 1);
    map.at(height - 1).at(width - 1) = new Node(height - 1, width - 1, 0);
    if (hard) {
        for (int j = height - 30; j < height - 10; j++) {
            for (int i = 30; i < width; i++) {
                delete map.at(i).at(j);
                map.at(i).at(j) = new Node(i, j, 1);
            }
        }
    }
    return map;
}

void deleteMap(std::vector<std::vector<Node*>>* map) {
    for (auto &i: *map) {
        for (auto &j: i) {
            delete j;
        }
    }
}

float density(std::vector<std::vector<Node*>>* map) {
    int obstacleCount = 0;
    for (auto &i: *map) {
        for (auto &j: i) {
            if (j->getObstacle()) {
                obstacleCount++;
            }
        }
    }
    return (float) obstacleCount / (float) (map->size() * map[0].size());
}